@@ -49,13 +49,17 @@ static gint     gail_focus_idle_handler  (gpointer             data);
 static void     gail_focus_notify        (GtkWidget            *widget);
 static void     gail_focus_notify_when_idle (GtkWidget            *widget);
 
-static void     gail_focus_tracker_init ();
+static void     gail_focus_tracker_init (void);
+static void     gail_focus_object_destroyed (gpointer data);
+static void     gail_focus_tracker (AtkObject *object);
 
 static GtkWidget* focus_widget = NULL;
 static GtkWidget* next_focus_widget = NULL;
 static GtkWidget* subsequent_focus_widget = NULL;
 static GtkWidget* focus_before_menu = NULL;
 static guint focus_notify_handler = 0;    
+static guint focus_tracker_id = 0;
+static GQuark quark_focus_object = 0;
 
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_WIDGET, gail_widget, gail_widget_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CONTAINER, gail_container, gail_container_new)
@@ -93,6 +97,7 @@ GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_OPTION_MENU, gail_option_menu, gail_option_me
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CANVAS, gail_canvas, gail_canvas_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CHECK_MENU_ITEM, gail_check_menu_item, gail_check_menu_item_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_RADIO_MENU_ITEM, gail_radio_menu_item, gail_radio_menu_item_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_EXPANDER, gail_expander, gail_expander_new)
 
 static AtkObject*
 gail_get_accessible_for_widget (GtkWidget *widget,
@@ -146,7 +151,7 @@ gail_get_accessible_for_widget (GtkWidget *widget,
       AtkObject *focus_object;
 
       obj = gtk_widget_get_accessible (widget);
-      focus_object = g_object_get_data (G_OBJECT (obj), "gail-focus-object");
+      focus_object = g_object_get_qdata (G_OBJECT (obj), quark_focus_object);
       if (focus_object)
         obj = focus_object;
     }
@@ -641,6 +646,63 @@ gail_focus_tracker_init ()
     }
 }
 
+static void
+gail_focus_object_destroyed (gpointer data)
+{
+  g_object_set_qdata (G_OBJECT (data), quark_focus_object, NULL);
+}
+
+static void
+gail_focus_tracker (AtkObject *focus_object)
+{
+  if (focus_object)
+    {
+      AtkObject *old_focus_object;
+
+      if (!GTK_IS_ACCESSIBLE (focus_object))
+        {
+          AtkObject *parent;
+
+          parent = focus_object;
+          while (1)
+            {
+              parent = atk_object_get_parent (parent);
+              if (parent == NULL)
+                break;
+              if (GTK_IS_ACCESSIBLE (parent))
+                break;
+            }
+
+          if (parent)
+            {
+              old_focus_object = g_object_get_qdata (G_OBJECT (parent), quark_focus_object);
+              if (old_focus_object != focus_object)
+                {
+                  if (old_focus_object)
+                    g_object_weak_unref (G_OBJECT (old_focus_object),
+                                         (GWeakNotify) gail_focus_object_destroyed,
+                                         parent);
+                  g_object_weak_ref (G_OBJECT (focus_object), 
+                                     (GWeakNotify) gail_focus_object_destroyed,
+                                     parent);
+                  g_object_set_qdata (G_OBJECT (parent), quark_focus_object, focus_object);
+                }
+            }
+        }
+      else
+        {
+          old_focus_object = g_object_get_qdata (G_OBJECT (focus_object), quark_focus_object);
+          if (old_focus_object)
+            {
+              g_object_weak_unref (G_OBJECT (old_focus_object),
+                                   (GWeakNotify) gail_focus_object_destroyed,
+                                   focus_object);
+              g_object_set_qdata (G_OBJECT (focus_object), quark_focus_object, NULL);
+            }
+        }
+    }
+}
+
 /*
  *   These exported symbols are hooked by gnome-program
  * to provide automatic module initialization and shutdown.
@@ -658,6 +720,7 @@ gail_accessibility_module_init (void)
       return;
     }
   gail_initialized = TRUE;
+  quark_focus_object = g_quark_from_static_string ("gail-focus-object");
 
   fprintf (stderr, "GTK Accessibility Module initialized\n");
 
@@ -700,6 +763,7 @@ gail_accessibility_module_init (void)
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_OPTION_MENU, gail_option_menu);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CHECK_MENU_ITEM, gail_check_menu_item);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_RADIO_MENU_ITEM, gail_radio_menu_item);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_EXPANDER, gail_expander);
 
   /* LIBGNOMECANVAS SUPPORT */
   GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS, gail_canvas);
@@ -711,6 +775,7 @@ gail_accessibility_module_init (void)
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_OBJECT, gail_object);
 
   atk_focus_tracker_init (gail_focus_tracker_init);
+  focus_tracker_id = atk_add_focus_tracker (gail_focus_tracker);
 
   /* Initialize the GailUtility class */
   g_type_class_unref (g_type_class_ref (GAIL_TYPE_UTIL));
@@ -744,6 +809,7 @@ gnome_accessibility_module_shutdown (void)
       return;
     }
   gail_initialized = FALSE;
+  atk_remove_focus_tracker (focus_tracker_id);
 
   fprintf (stderr, "GTK Accessibility Module shutdown\n");
 
