@@ -384,6 +384,11 @@ gail_entry_finalize (GObject            *object)
       g_source_remove (entry->action_idle_handler);
       entry->action_idle_handler = 0;
     }
+  if (entry->insert_idle_handler)
+    {
+      g_source_remove (entry->insert_idle_handler);
+      entry->insert_idle_handler = 0;
+    }
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -1055,6 +1060,7 @@ idle_notify_insert (gpointer data)
 {
   GailEntry *entry = GAIL_ENTRY (data);
 
+  entry->insert_idle_handler = 0;
   gail_entry_notify_insert (entry);
   return FALSE;
 }
@@ -1096,7 +1102,8 @@ _gail_entry_insert_text_cb (GtkEntry *entry,
    * The signal will be emitted when the cursor position is updated.
    * or in an idle handler if it not updated.
    */
-   g_idle_add (idle_notify_insert, gail_entry);
+   if (gail_entry->insert_idle_handler)
+     gail_entry->insert_idle_handler = g_idle_add (idle_notify_insert, gail_entry);
 }
 
 static gunichar 
