@@ -27,9 +27,10 @@ static G_CONST_RETURN gchar* gail_item_get_name        (AtkObject     *obj);
 static gint                  gail_item_get_n_children  (AtkObject     *obj);
 static AtkObject*            gail_item_ref_child       (AtkObject     *obj,
                                                         gint          i);
-
 static void                  gail_item_real_initialize (AtkObject     *obj,
                                                         gpointer      data);
+static gint                  gail_item_label_map_gtk   (GtkWidget     *widget,
+                                                        gpointer      data);
 static void                  gail_item_finalize        (GObject       *object);
 static void                  gail_item_init_textutil   (GailItem      *item,
                                                         GtkWidget     *label);
@@ -172,27 +173,47 @@ gail_item_real_initialize (AtkObject *obj,
 
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
+  item->textutil = NULL;
+
   label = get_label_from_item (GTK_WIDGET (data));
   if (GTK_IS_LABEL (label))
-    {
+    if (GTK_WIDGET_MAPPED (label))
       gail_item_init_textutil (item, label);
-    }
+    else
+      g_signal_connect (label,
+                        "map",
+                        G_CALLBACK (gail_item_label_map_gtk),
+                        item);
+
   obj->role = ATK_ROLE_LIST_ITEM;
 }
 
+static gint
+gail_item_label_map_gtk (GtkWidget *widget,
+                         gpointer data)
+{
+  GailItem *item;
+
+  item = GAIL_ITEM (data);
+  gail_item_init_textutil (item, widget);
+}
+
 static void
 gail_item_init_textutil (GailItem  *item,
                          GtkWidget *label)
 {
   const gchar *label_text;
 
-  item->textutil = gail_text_util_new ();
+  if (item->textutil == NULL)
+    {
+      item->textutil = gail_text_util_new ();
+      g_signal_connect (label,
+                        "notify",
+                        (GCallback) gail_item_notify_label_gtk,
+                        item);     
+    }
   label_text = gtk_label_get_text (GTK_LABEL (label));
   gail_text_util_text_setup (item->textutil, label_text);
-  g_signal_connect (label,
-                    "notify",
-                    (GCallback) gail_item_notify_label_gtk,
-                    item);     
 }
 
 static void
