@@ -20,6 +20,7 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include "gaillabel.h"
+#include "gailwindow.h"
 #include <libgail-util/gailmisc.h>
 
 static void       gail_label_class_init            (GailLabelClass    *klass);
@@ -27,6 +28,10 @@ static void	  gail_label_real_initialize	   (AtkObject 	      *obj,
                                                     gpointer	      data);
 static void	  gail_label_real_notify_gtk	   (GObject	      *obj,
                                                     GParamSpec	      *pspec);
+static gint       gail_label_map_gtk               (GtkWidget         *widget,
+                                                    gpointer          data);
+static void       gail_label_init_text_util        (GailLabel         *gail_label,
+                                                    GtkWidget         *widget);
 static void       gail_label_finalize              (GObject           *object);
 
 static void       atk_text_interface_init          (AtkTextIface      *iface);
@@ -161,7 +166,6 @@ gail_label_real_initialize (AtkObject *obj,
   GtkLabel  *label;
   GtkWidget  *widget;
   GailLabel *gail_label;
-  const gchar *label_text;
 
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
   
@@ -171,23 +175,23 @@ gail_label_real_initialize (AtkObject *obj,
   gail_label->has_top_level = FALSE;
   gail_label->cursor_position = 0;
   gail_label->selection_bound = 0;
+  gail_label->textutil = NULL;
+  gail_label->label_length = 0;
   
-  gail_label->textutil = gail_text_util_new ();
+  widget = GTK_WIDGET (data);
 
-  label = GTK_LABEL (data);
-  label_text = gtk_label_get_text (label);
-  gail_text_util_text_setup (gail_label->textutil, label_text);
-  
-  if (label_text == NULL)
-    gail_label->label_length = 0;
+  if (GTK_WIDGET_MAPPED (widget))
+    gail_label_init_text_util (gail_label, widget);
   else
-    gail_label->label_length = g_utf8_strlen (label_text, -1);
+    g_signal_connect (widget,
+                      "map",
+                      G_CALLBACK (gail_label_map_gtk),
+                      gail_label);
 
   /* 
    * Check whether ancestor of GtkLabel is a GtkButton and if so
    * set accessible parent for GailLabel
    */
-  widget = GTK_WIDGET (data);
   while (widget != NULL)
     {
       widget = gtk_widget_get_parent (widget);
@@ -204,6 +208,36 @@ gail_label_real_initialize (AtkObject *obj,
     obj->role = ATK_ROLE_LABEL;
 }
 
+static gint
+gail_label_map_gtk (GtkWidget *widget,
+                    gpointer data)
+{
+  GailLabel *gail_label;
+
+  gail_label = GAIL_LABEL (data);
+  gail_label_init_text_util (gail_label, widget);
+}
+
+static void
+gail_label_init_text_util (GailLabel *gail_label,
+                           GtkWidget *widget)
+{
+  GtkLabel *label;
+  const gchar *label_text;
+
+  if (gail_label->textutil == NULL)
+    gail_label->textutil = gail_text_util_new ();
+
+  label = GTK_LABEL (widget);
+  label_text = gtk_label_get_text (label);
+  gail_text_util_text_setup (gail_label->textutil, label_text);
+  
+  if (label_text == NULL)
+    gail_label->label_length = 0;
+  else
+    gail_label->label_length = g_utf8_strlen (label_text, -1);
+}
+
 AtkObject* 
 gail_label_new (GtkWidget *widget)
 {
@@ -227,7 +261,6 @@ notify_name_change (AtkObject *atk_obj)
   GailLabel *gail_label;
   GtkWidget *widget;
   GObject *gail_obj;
-  const gchar* label_text;
 
   widget = GTK_ACCESSIBLE (atk_obj)->widget;
   if (widget == NULL)
@@ -242,15 +275,11 @@ notify_name_change (AtkObject *atk_obj)
 
   /* Create a delete text and an insert text signal */
  
-  label_text = gtk_label_get_text (label);
-
   g_signal_emit_by_name (gail_obj, "text_changed::delete", 0, 
                          gail_label->label_length);
-  gail_text_util_text_setup (gail_label->textutil, label_text);
-  if (label_text == NULL)
-    gail_label->label_length = 0;
-  else
-    gail_label->label_length = g_utf8_strlen (label_text, -1);
+
+  gail_label_init_text_util (gail_label, widget);
+
   g_signal_emit_by_name (gail_obj, "text_changed::insert", 0, 
                          gail_label->label_length);
 
@@ -305,7 +334,8 @@ gail_label_real_notify_gtk (GObject           *obj,
             }
           if (atk_object_get_role (top_level) != ATK_ROLE_APPLICATION)
             {
-              if (gail_label->window_create_handler == 0)
+              if (gail_label->window_create_handler == 0 && 
+                  GAIL_IS_WINDOW (top_level))
                 gail_label->window_create_handler = g_signal_connect_after (top_level, "create", G_CALLBACK (window_created), atk_obj);
             }
           else
@@ -378,7 +408,8 @@ gail_label_finalize (GObject            *object)
 {
   GailLabel *label = GAIL_LABEL (object);
 
-  g_object_unref (label->textutil);
+  if (label->textutil)
+    g_object_unref (label->textutil);
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
