@@ -136,11 +136,12 @@ gail_menu_get_parent (AtkObject *accessible)
       g_return_val_if_fail (GTK_IS_MENU (widget), NULL);
 
       /*
-       * If the menu is attached to a menu item report the menu item as parent
+       * If the menu is attached to a menu item or a button (Gnome Menu)
+       * report the menu item as parent.
        */
       parent_widget = gtk_menu_get_attach_widget (GTK_MENU (widget));
 
-      if (!GTK_IS_MENU_ITEM (parent_widget))
+      if (!GTK_IS_MENU_ITEM (parent_widget) && !GTK_IS_BUTTON (parent_widget))
         parent_widget = widget->parent;
 
       if (parent_widget == NULL)
