@@ -40,13 +40,15 @@ typedef struct _GailRadioButtonClass         GailRadioButtonClass;
 struct _GailRadioButton
 {
   GailToggleButton parent;
+
+  GSList *old_group;
 };
 
 GType gail_radio_button_get_type (void);
 
 struct _GailRadioButtonClass
 {
-  GailButtonClass parent_class;
+  GailToggleButtonClass parent_class;
 };
 
 AtkObject* gail_radio_button_new (GtkWidget *widget);
